Merge branch 't/test_man'
[paraslash.git] / write.h
diff --git a/write.h b/write.h
index 9723977ae4357506ac1c6b64284695577d1e5683..0cfcafdade93f8334a9b8236656ba967def148ec 100644 (file)
--- a/write.h
+++ b/write.h
@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008 Andre Noll <maan@systemlinux.org>
+ * Copyright (C) 2006-2014 Andre Noll <maan@systemlinux.org>
  *
  * Licensed under the GPL v2. For licencing details see COPYING.
  */
@@ -13,18 +13,18 @@ enum writer_enum {WRITER_ENUM};
  * Describes one running instance of a writer.
  */
 struct writer_node {
-       /** Points to the writer structure associated with this node. */
-       struct writer *writer; /* FIXME: Should better be only the number. */
+       /** The number of this writer. */
+       int writer_num;
        /** Writer-specific data. */
        void *private_data;
-       /** Send that many bytes in one go. */
-       int chunk_bytes;
-       /** Pointer to the group this node belongs to. */
-       struct writer_node_group *wng;
        /** The writer-specific configuration of this node. */
        void *conf;
-       /** How much of the wng's buffer is already written. */
-       size_t written;
+       /** The buffer tree node associated with this writer node. */
+       struct btr_node *btrn;
+       /** The task of this writer node. */
+       struct task task;
+       /** The minimal input queue size (size of one audio sample). */
+       size_t min_iqs;
 };
 
 /** Describes one supported writer. */
@@ -39,81 +39,50 @@ struct writer {
        /**
         * The command line parser of the writer.
         *
* It should check whether the command line options given by \a options are
* valid.  On success, it should return a pointer to the writer-specific
* configuration data determined by \a options.  Note that this might be called
* more than once with different values of \a options.
         */
-       void *(*parse_config)(const char *options);
+       void *(*parse_config_or_die)(int argc, char **argv);
        /**
-        * Open one instance of this writer.
+        * Dellocate all configuration resources.
         *
-        * This function should perform any work necessary to write the incoming
-        * stream. If To this aim, it may allocate its private data structure and store
-        * a pointer to that structure via the given writer_node parameter.
+        * This should free whatever was allocated by \ref parse_config_or_die().
         */
-       int (*open)(struct writer_node *);
+       void (*free_config)(void *config);
        /**
         * Prepare the fd sets for select.
         *
-        * This is called from the writer node group task's pre_select(). It
-        * may use the sched pointer to add any file descriptors or to decrease
-        * the select timeout. It must return positive on success and negative
-        * on errors.
+        * This is called from scheduler. It may use the sched pointer to add
+        * any file descriptors or to decrease the select timeout.
         */
-       int (*pre_select)(struct sched *s, struct writer_node *wn);
+       void (*pre_select)(struct sched *s, struct task *t);
        /**
         * Write audio data.
         *
-        * Called from the post_select function of the wng task. It must keep
-        * track of the number of bytes consumed from the wng's buffer via
-        * the \p wn->written variable (which may be modified by the wng handling
-        * functions). This function must return positive on success and
-        * negative on errors.
+        * Called from the post_select function of the writer node's task.
         */
-       int (*post_select)(struct sched *s, struct writer_node *wn);
+       int (*post_select)(struct sched *s, struct task *t);
        /**
         * Close one instance of the writer.
         *
         * This function is assumed to succeed.
         */
        void (*close)(struct writer_node *);
+       /** The short and the log help text of this writer. */
+       struct ggo_help help;
        /**
-        * Shutdown the writer
+        * The callback handler.
         *
-        * This is a optional function pointer used for cleaning up.
+        * Each writer may provide an ->execute callback which can be used for
+        * inter-node communication.
         */
-       void (*shutdown)(struct writer_node *);
-};
-
-/**
- * Describes a set of writer nodes that all write the same stream.
- */
-struct writer_node_group {
-       /** Number of nodes belonging to this group. */
-       unsigned num_writers;
-       /** Array of pointers to the corresponding writer nodes. */
-       struct writer_node *writer_nodes;
-       /** The maximum of the chunk_bytes values of the writer nodes in this group. */
-       int max_chunk_bytes;
-       /** Non-zero if an error or end of file was encountered by the feeding task. */
-       int *input_error;
-       /** Current output buffer. */
-       char *buf;
-       /** Number of bytes loaded in the output buffer. */
-       size_t *loaded;
-       /** Number of audio channels of the current stream. */
-       unsigned int *channels;
-       /** Sample rate of the current stream. */
-       unsigned int *samplerate;
-       /** The task associated to this group. */
-       struct task task;
-       /** Whether the group is open, i.e. wng_open() was called. */
-       int open;
+       btr_command_handler execute;
 };
 
-/** Loop over each writer node in a writer group. */
-#define FOR_EACH_WRITER_NODE(i, wng) for (i = 0; i < (wng)->num_writers; i++)
 /** Loop over each supported writer. */
 #define FOR_EACH_WRITER(i) for (i = 0; i < NUM_SUPPORTED_WRITERS; i++)
@@ -39,81 +39,50 @@ struct writer {
         * The command line parser of the writer.
         *
-        * It should check whether the command line options given by \a options are
-        * valid.  On success, it should return a pointer to the writer-specific
-        * configuration data determined by \a options.  Note that this might be called
-        * more than once with different values of \a options.
+        * It should check whether the command line options given by \a argv
+        * and \a argc are valid and return a pointer to the writer-specific
+        * configuration data determined by these options. This function must
+        * either succeed or call exit(). Note that parse_config_or_die() might
+        * be called more than once with different values of \a options. \sa
+        * \ref free_config().
         */